@@ -1,62 +1,44 @@
-// Copyright (c) 2018-2020 Sean McArthur
-// Licensed under the MIT license http://opensource.org/licenses/MIT
-// modified from https://github.com/seanmonstar/salvo/blob/master/src/filters/cors.rs
-//! CORS Filters
+//! [CORS]: https://developer.mozilla.org/en-US/docs/Web/HTTP/CORS
+//!
+//! # Example
+//!
+//! ```
+//! use salvo_core::prelude::*;
+//! use salvo_extra::cors::CorsHandler;
+//!
+//! let cors_handler = CorsHandler::builder()
+//!     .with_allow_origin("https://salvo.rs")
+//!     .with_allow_methods(vec!["GET", "POST", "DELETE"]).build();
+//!
+//! let router = Router::new().before(cors_handler).post(upload_file);
+//! #[fn_handler]
+//! async fn upload_file(res: &mut Response) {
+//! }
+//!
+//! ```
+//! If you want to allow any route:
+//! ```
+//! use salvo_core::prelude::*;
+//! use salvo_extra::cors::CorsHandler;
+//! let cors_handler = CorsHandler::builder()
+//!     .with_allow_any_origin().build();
+//! ```
 
 use std::collections::HashSet;
 use std::convert::TryFrom;
 use std::error::Error as StdError;
-use std::sync::Arc;
 
 use async_trait::async_trait;
 use salvo_core::http::header::{self, HeaderMap, HeaderName, HeaderValue};
 use salvo_core::http::headers::{
     AccessControlAllowHeaders, AccessControlAllowMethods, AccessControlExposeHeaders, HeaderMapExt, Origin,
 };
-use salvo_core::http::Method;
-use salvo_core::http::{Request, Response};
+use salvo_core::http::{Method, Request, Response, StatusCode};
 use salvo_core::{Depot, Handler};
 
-/// [CORS]: https://developer.mozilla.org/en-US/docs/Web/HTTP/CORS
-///
-/// # Example
-///
-/// ```
-/// use salvo_core::prelude::*;
-///
-/// let cors = salvo_extra::cors::cors()
-///     .allow_origin("https://hyper.rs")
-///     .allow_methods(vec!["GET", "POST", "DELETE"]).build();
-/// let cors = salvo_extra::cors::cors().allow_origin("https://hyper.rs")
-///     .allow_methods(vec!["GET", "POST", "DELETE"]).build();
-///
-/// let router = Router::new().before(cors).post(upload_file);
-/// #[fn_handler]
-/// async fn upload_file(res: &mut Response) {
-/// }
-///
-/// ```
-/// If you want to allow any route:
-/// ```
-/// use salvo_core::prelude::*;
-/// let cors = salvo_extra::cors::cors()
-///     .allow_any_origin();
-/// ```
-/// You can find more usage examples [here](https://github.com/salvo-rs/salvo/blob/examples/cors.rs).
-pub fn cors() -> Builder {
-    Builder {
-        credentials: false,
-        allowed_headers: HashSet::new(),
-        exposed_headers: HashSet::new(),
-        max_age: None,
-        methods: HashSet::new(),
-        origins: None,
-    }
-}
-
-/// A constructed via `salvo_extra::cors::cors()`.
+/// A constructed via `salvo_extra::cors::CorsHandler::builder()`.
 #[derive(Clone, Debug)]
-pub struct Builder {
+pub struct HandlerBuilder {
     credentials: bool,
     allowed_headers: HashSet<HeaderName>,
     exposed_headers: HashSet<HeaderName>,
@@ -64,10 +46,26 @@ pub struct Builder {
     methods: HashSet<Method>,
     origins: Option<HashSet<HeaderValue>>,
 }
+impl Default for HandlerBuilder {
+    fn default() -> Self {
+        Self::new()
+    }
+}
 
-impl Builder {
+impl HandlerBuilder {
+    pub fn new() -> Self {
+        HandlerBuilder {
+            credentials: false,
+            allowed_headers: HashSet::new(),
+            exposed_headers: HashSet::new(),
+            max_age: None,
+            methods: HashSet::new(),
+            origins: None,
+        }
+    }
     /// Sets whether to add the `Access-Control-Allow-Credentials` header.
-    pub fn allow_credentials(mut self, allow: bool) -> Self {
+    #[inline]
+    pub fn with_allow_credentials(mut self, allow: bool) -> Self {
         self.credentials = allow;
         self
     }
@@ -77,7 +75,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if the provided argument is not a valid `http::Method`.
-    pub fn allow_method<M>(mut self, method: M) -> Self
+    #[inline]
+    pub fn with_allow_method<M>(mut self, method: M) -> Self
     where
         Method: TryFrom<M>,
     {
@@ -94,7 +93,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if the provided argument is not a valid `http::Method`.
-    pub fn allow_methods<I>(mut self, methods: I) -> Self
+    #[inline]
+    pub fn with_allow_methods<I>(mut self, methods: I) -> Self
     where
         I: IntoIterator,
         Method: TryFrom<I::Item>,
@@ -114,7 +114,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if the provided argument is not a valid `http::header::HeaderName`.
-    pub fn allow_header<H>(mut self, header: H) -> Self
+    #[inline]
+    pub fn with_allow_header<H>(mut self, header: H) -> Self
     where
         HeaderName: TryFrom<H>,
     {
@@ -133,7 +134,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if any of the headers are not a valid `http::header::HeaderName`.
-    pub fn allow_headers<I>(mut self, headers: I) -> Self
+    #[inline]
+    pub fn with_allow_headers<I>(mut self, headers: I) -> Self
     where
         I: IntoIterator,
         HeaderName: TryFrom<I::Item>,
@@ -151,7 +153,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if the provided argument is not a valid `http::header::HeaderName`.
-    pub fn expose_header<H>(mut self, header: H) -> Self
+    #[inline]
+    pub fn with_expose_header<H>(mut self, header: H) -> Self
     where
         HeaderName: TryFrom<H>,
     {
@@ -168,7 +171,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if any of the headers are not a valid `http::header::HeaderName`.
-    pub fn expose_headers<I>(mut self, headers: I) -> Self
+    #[inline]
+    pub fn with_expose_headers<I>(mut self, headers: I) -> Self
     where
         I: IntoIterator,
         HeaderName: TryFrom<I::Item>,
@@ -187,7 +191,8 @@ impl Builder {
     ///
     /// This can allow websites you didn't intend to access this resource,
     /// it is usually better to set an explicit list.
-    pub fn allow_any_origin(mut self) -> Self {
+    #[inline]
+    pub fn with_allow_any_origin(mut self) -> Self {
         self.origins = None;
         self
     }
@@ -197,8 +202,9 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if the provided argument is not a valid `Origin`.
-    pub fn allow_origin(self, origin: impl IntoOrigin) -> Self {
-        self.allow_origins(Some(origin))
+    #[inline]
+    pub fn with_allow_origin(self, origin: impl IntoOrigin) -> Self {
+        self.with_allow_origins(Some(origin))
     }
 
     /// Add multiple origins to the existing list of allowed `Origin`s.
@@ -206,7 +212,8 @@ impl Builder {
     /// # Panics
     ///
     /// Panics if the provided argument is not a valid `Origin`.
-    pub fn allow_origins<I>(mut self, origins: I) -> Self
+    #[inline]
+    pub fn with_allow_origins<I>(mut self, origins: I) -> Self
     where
         I: IntoIterator,
         I::Item: IntoOrigin,
@@ -232,18 +239,19 @@ impl Builder {
     /// use std::time::Duration;
     /// use salvo_core::prelude::*;;
     ///
-    /// let cors = salvo_extra::cors::cors()
-    ///     .max_age(30) // 30u32 seconds
-    ///     .max_age(Duration::from_secs(30)); // or a Duration
+    /// let cors = salvo_extra::cors::CorsHandler::builder()
+    ///     .with_max_age(30) // 30u32 seconds
+    ///     .with_max_age(Duration::from_secs(30)); // or a Duration
     /// ```
-    pub fn max_age(mut self, seconds: impl Seconds) -> Self {
+    #[inline]
+    pub fn with_max_age(mut self, seconds: impl Seconds) -> Self {
         self.max_age = Some(seconds.seconds());
         self
     }
 
     /// Builds the `Cors` wrapper from the configured settings.
     ///
-    /// This step isn't *required*, as the `Builder` itself can be passed
+    /// This step isn't *required*, as the `HandlerBuilder` itself can be passed
     /// to `Filter::with`. This just allows constructing once, thus not needing
     /// to pay the cost of "building" every time.
     pub fn build(self) -> CorsHandler {
@@ -255,14 +263,26 @@ impl Builder {
         let allowed_headers_header = self.allowed_headers.iter().cloned().collect();
         let methods_header = self.methods.iter().cloned().collect();
 
-        let config = Arc::new(Configured {
-            cors: self,
+        let HandlerBuilder {
+            credentials,
+            allowed_headers,
+            exposed_headers,
+            max_age,
+            methods,
+            origins,
+        } = self;
+
+        CorsHandler {
+            credentials,
+            allowed_headers,
+            exposed_headers,
+            max_age,
+            methods,
+            origins,
             allowed_headers_header,
             expose_headers_header,
             methods_header,
-        });
-
-        CorsHandler { config }
+        }
     }
 }
 
@@ -291,26 +311,33 @@ impl ::std::fmt::Display for Forbidden {
 
 impl StdError for Forbidden {}
 
-#[derive(Clone, Debug)]
-struct Configured {
-    cors: Builder,
-    allowed_headers_header: AccessControlAllowHeaders,
-    expose_headers_header: Option<AccessControlExposeHeaders>,
-    methods_header: AccessControlAllowMethods,
-}
-
 enum Validated {
     Preflight(HeaderValue),
     Simple(HeaderValue),
     NotCors,
 }
 
-impl Configured {
+#[derive(Debug)]
+pub struct CorsHandler {
+    credentials: bool,
+    allowed_headers: HashSet<HeaderName>,
+    exposed_headers: HashSet<HeaderName>,
+    max_age: Option<u64>,
+    methods: HashSet<Method>,
+    origins: Option<HashSet<HeaderValue>>,
+    allowed_headers_header: AccessControlAllowHeaders,
+    expose_headers_header: Option<AccessControlExposeHeaders>,
+    methods_header: AccessControlAllowMethods,
+}
+impl CorsHandler {
+    #[inline]
+    pub fn builder() -> HandlerBuilder {
+        HandlerBuilder::default()
+    }
     fn check_request(&self, method: &Method, headers: &HeaderMap) -> Result<Validated, Forbidden> {
         match (headers.get(header::ORIGIN), method) {
             (Some(origin), &Method::OPTIONS) => {
                 // OPTIONS requests are preflight CORS requests...
-
                 if !self.is_origin_allowed(origin) {
                     return Err(Forbidden::Origin);
                 }
@@ -337,7 +364,6 @@ impl Configured {
             }
             (Some(origin), _) => {
                 // Any other method, simply check for a valid origin...
-
                 tracing::debug!("origin header: {:?}", origin);
                 if self.is_origin_allowed(origin) {
                     Ok(Validated::Simple(origin.clone()))
@@ -352,20 +378,23 @@ impl Configured {
         }
     }
 
+    #[inline]
     fn is_method_allowed(&self, header: &HeaderValue) -> bool {
         Method::from_bytes(header.as_bytes())
-            .map(|method| self.cors.methods.contains(&method))
+            .map(|method| self.methods.contains(&method))
             .unwrap_or(false)
     }
 
+    #[inline]
     fn is_header_allowed(&self, header: &str) -> bool {
         HeaderName::from_bytes(header.as_bytes())
-            .map(|header| self.cors.allowed_headers.contains(&header))
+            .map(|header| self.allowed_headers.contains(&header))
             .unwrap_or(false)
     }
 
+    #[inline]
     fn is_origin_allowed(&self, origin: &HeaderValue) -> bool {
-        if let Some(ref allowed) = self.cors.origins {
+        if let Some(ref allowed) = self.origins {
             allowed.contains(origin)
         } else {
             true
@@ -378,13 +407,13 @@ impl Configured {
         headers.typed_insert(self.allowed_headers_header.clone());
         headers.typed_insert(self.methods_header.clone());
 
-        if let Some(max_age) = self.cors.max_age {
+        if let Some(max_age) = self.max_age {
             headers.insert(header::ACCESS_CONTROL_MAX_AGE, max_age.into());
         }
     }
 
     fn append_common_headers(&self, headers: &mut HeaderMap) {
-        if self.cors.credentials {
+        if self.credentials {
             headers.insert(
                 header::ACCESS_CONTROL_ALLOW_CREDENTIALS,
                 HeaderValue::from_static("true"),
@@ -396,27 +425,23 @@ impl Configured {
     }
 }
 
-#[derive(Debug)]
-pub struct CorsHandler {
-    config: Arc<Configured>,
-}
-
 #[async_trait]
 impl Handler for CorsHandler {
     async fn handle(&self, req: &mut Request, _depot: &mut Depot, res: &mut Response) {
-        let validated = self.config.check_request(req.method(), req.headers());
+        let validated = self.check_request(req.method(), req.headers());
 
         match validated {
             Ok(Validated::Preflight(origin)) => {
-                self.config.append_preflight_headers(res.headers_mut());
+                self.append_preflight_headers(res.headers_mut());
                 res.headers_mut().insert(header::ACCESS_CONTROL_ALLOW_ORIGIN, origin);
             }
             Ok(Validated::Simple(origin)) => {
-                self.config.append_common_headers(res.headers_mut());
+                self.append_common_headers(res.headers_mut());
                 res.headers_mut().insert(header::ACCESS_CONTROL_ALLOW_ORIGIN, origin);
             }
             Err(err) => {
                 tracing::error!(error = %err, "CorsHandler validate error");
+                res.set_status_code(StatusCode::FORBIDDEN);
             }
             _ => {}
         }
@@ -428,12 +453,14 @@ pub trait Seconds {
 }
 
 impl Seconds for u32 {
+    #[inline]
     fn seconds(self) -> u64 {
         self.into()
     }
 }
 
 impl Seconds for ::std::time::Duration {
+    #[inline]
     fn seconds(self) -> u64 {
         self.as_secs()
     }
@@ -452,3 +479,94 @@ impl<'a> IntoOrigin for &'a str {
         Origin::try_from_parts(scheme, rest, None).expect("invalid Origin")
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use salvo_core::http::header::*;
+    use salvo_core::hyper;
+    use salvo_core::prelude::*;
+
+    use super::*;
+
+    #[tokio::test]
+    async fn test_cors() {
+        let cors_handler = CorsHandler::builder()
+            .with_allow_origin("https://salvo.rs")
+            .with_allow_methods(vec!["GET", "POST", "OPTIONS"])
+            .with_allow_headers(vec![
+                "CONTENT-TYPE",
+                "Access-Control-Request-Method",
+                "Access-Control-Allow-Origin",
+                "Access-Control-Allow-Headers",
+                "Access-Control-Max-Age",
+            ])
+            .build();
+
+        #[fn_handler]
+        async fn hello() -> &'static str {
+            "hello"
+        }
+
+        let router = Router::new()
+            .before(cors_handler)
+            .push(Router::with_path("hello").handle(hello));
+        let service = Service::new(router);
+
+        async fn options_access(service: &Service, origin: &str) -> Response {
+            let request = Request::from_hyper(
+                hyper::Request::builder()
+                    .method("OPTIONS")
+                    .uri("http://127.0.0.1:7979/hello")
+                    .header("Origin", origin)
+                    .header("Access-Control-Request-Method", "POST")
+                    .header("Access-Control-Request-Headers", "Content-Type")
+                    .body(hyper::Body::empty())
+                    .unwrap(),
+            );
+            service.handle(request).await
+        }
+
+        async fn access(service: &Service, method: &str, origin: &str) -> Response {
+            let request = Request::from_hyper(
+                hyper::Request::builder()
+                    .method(method)
+                    .uri("http://127.0.0.1:7979/hello")
+                    .header("Origin", origin)
+                    .body(hyper::Body::empty())
+                    .unwrap(),
+            );
+            service.handle(request).await
+        }
+
+        let response = access(&service, "OPTIONS", "https://salvo.rs").await;
+        let headers = response.headers();
+        assert!(headers.get(ACCESS_CONTROL_ALLOW_METHODS).is_none());
+
+        let response = options_access(&service, "https://salvo.rs").await;
+        let headers = response.headers();
+        assert!(headers.get(ACCESS_CONTROL_ALLOW_METHODS).is_some());
+        assert!(headers.get(ACCESS_CONTROL_ALLOW_HEADERS).is_some());
+
+        let response = access(&service, "OPTIONS", "https://google.com").await;
+        let headers = response.headers();
+        assert!(
+            headers.get(ACCESS_CONTROL_ALLOW_METHODS).is_none(),
+            "POST, GET, DELETE, OPTIONS"
+        );
+        assert!(headers.get(ACCESS_CONTROL_ALLOW_HEADERS).is_none());
+
+        let content = access(&service, "GET", "https://salvo.rs")
+            .await
+            .take_text()
+            .await
+            .unwrap();
+        assert!(content.contains("hello"));
+
+        let content = access(&service, "GET", "https://google.rs")
+            .await
+            .take_text()
+            .await
+            .unwrap();
+        assert!(content.contains("Forbidden"));
+    }
+}
