@@ -7,7 +7,8 @@
 //! ```no_run
 //! use std::time::Duration;
 //! use std::convert::Infallible;
-//! use futures::{stream::iter, Stream};
+//! use futures_util::stream::iter;
+//! use futures_util::Stream;
 //!
 //! use salvo_core::prelude::*;
 //! use salvo_extra::sse::{self, SseEvent};
@@ -33,7 +34,7 @@
 //! }
 //! #[tokio::main]
 //! async fn main() {
-//!     let router = Router::new().path("push-notifications").get(handle);
+//!     let router = Router::with_path("push-notifications").get(handle);
 //!     Server::new(router).bind(([0, 0, 0, 0], 3131)).await;
 //! }
 //! ```
@@ -54,7 +55,7 @@ use std::pin::Pin;
 use std::task::{Context, Poll};
 use std::time::Duration;
 
-use futures::{future, Stream, TryStream, TryStreamExt};
+use futures_util::{future, Stream, TryStream, TryStreamExt};
 use pin_project::pin_project;
 use salvo_core::http::header::{HeaderValue, CACHE_CONTROL, CONTENT_TYPE};
 use serde_json::{self, Error};
@@ -203,7 +204,7 @@ impl Display for SseEvent {
 pub struct SseKeepAlive<S> {
     #[pin]
     event_stream: S,
-    comment_text: Cow<'static, str>,
+    comment: Cow<'static, str>,
     max_interval: Duration,
     #[pin]
     alive_timer: Sleep,
@@ -219,7 +220,7 @@ where
         let alive_timer = time::sleep(max_interval);
         SseKeepAlive {
             event_stream,
-            comment_text: Cow::Borrowed(""),
+            comment: Cow::Borrowed(""),
             max_interval,
             alive_timer,
         }
@@ -227,7 +228,7 @@ where
     /// Customize the interval between keep-alive messages.
     ///
     /// Default is 15 seconds.
-    pub fn interval(mut self, time: Duration) -> Self {
+    pub fn with_interval(mut self, time: Duration) -> Self {
         self.max_interval = time;
         self
     }
@@ -235,8 +236,8 @@ where
     /// Customize the text of the keep-alive message.
     ///
     /// Default is an empty comment.
-    pub fn text(mut self, text: impl Into<Cow<'static, str>>) -> Self {
-        self.comment_text = text.into();
+    pub fn with_comment(mut self, comment: impl Into<Cow<'static, str>>) -> Self {
+        self.comment = comment.into();
         self
     }
 
@@ -293,8 +294,7 @@ where
                 Poll::Ready(_) => {
                     // restart timer
                     pin.alive_timer.reset(tokio::time::Instant::now() + *pin.max_interval);
-                    let comment_str = pin.comment_text.clone();
-                    let event = SseEvent::default().comment(comment_str);
+                    let event = SseEvent::default().comment(pin.comment.clone());
                     Poll::Ready(Some(Ok(event)))
                 }
             },
@@ -311,3 +311,100 @@ where
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use std::convert::Infallible;
+    use std::time::Duration;
+
+    use salvo_core::prelude::*;
+    use tokio_stream;
+
+    use super::*;
+
+    #[tokio::test]
+    async fn test_sse_data() {
+        let event_stream = tokio_stream::iter(vec![
+            Ok::<_, Infallible>(SseEvent::default().data("1")),
+            Ok::<_, Infallible>(SseEvent::default().data("2")),
+        ]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains("data:1") && text.contains("data:2"));
+    }
+
+    #[tokio::test]
+    async fn test_sse_keep_alive() {
+        let event_stream = tokio_stream::iter(vec![Ok::<_, Infallible>(SseEvent::default().data("1"))]);
+        let mut response = Response::new();
+        SseKeepAlive::new(event_stream)
+            .with_comment("love you")
+            .with_interval(Duration::from_secs(1))
+            .streaming(&mut response);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains("data:1"));
+    }
+
+    #[tokio::test]
+    async fn test_sse_json() {
+        #[derive(Serialize, Debug)]
+        struct User {
+            name: String,
+        }
+
+        let event_stream = tokio_stream::iter(vec![SseEvent::default().json_data(User {
+            name: "jobs".to_owned(),
+        })]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains(r#"data:{"name":"jobs"}"#));
+    }
+
+    #[tokio::test]
+    async fn test_sse_comment() {
+        let event_stream = tokio_stream::iter(vec![Ok::<_, Infallible>(SseEvent::default().comment("comment"))]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains(":comment"));
+    }
+
+    #[tokio::test]
+    async fn test_sse_name() {
+        let event_stream = tokio_stream::iter(vec![Ok::<_, Infallible>(SseEvent::default().name("evt2"))]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains("event:evt2"));
+    }
+
+    #[tokio::test]
+    async fn test_sse_retry() {
+        let event_stream = tokio_stream::iter(vec![Ok::<_, Infallible>(
+            SseEvent::default().retry(std::time::Duration::from_secs_f32(1.02)),
+        )]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains("retry:1020"));
+
+        let event_stream = tokio_stream::iter(vec![Ok::<_, Infallible>(
+            SseEvent::default().retry(std::time::Duration::from_secs_f32(1.001)),
+        )]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains("retry:1001"));
+    }
+
+    #[tokio::test]
+    async fn test_sse_id() {
+        let event_stream = tokio_stream::iter(vec![Ok::<_, Infallible>(SseEvent::default().id("jobs"))]);
+        let mut response = Response::new();
+        super::streaming(&mut response, event_stream);
+        let text = response.take_text().await.unwrap();
+        assert!(text.contains("id:jobs"));
+    }
+}
