@@ -10,7 +10,7 @@ use std::future::Future;
 use std::pin::Pin;
 use std::task::{Context, Poll};
 
-use futures::{future, ready, FutureExt, Sink, Stream, TryFutureExt};
+use futures_util::{future, ready, FutureExt, Sink, Stream, TryFutureExt};
 use hyper::upgrade::OnUpgrade;
 use salvo_core::http::errors::*;
 use salvo_core::http::header::{SEC_WEBSOCKET_VERSION, UPGRADE};
